#include <can_vca.h>
#include <vcasdo_fsm.h>
#include <string.h>
#include <ul_gavl.h>
#include <ul_list.h>
#include <ul_evcbase.h>
#include <vca_net.h>
#include <ul_gavlcust.h>
#include <vca_pollfdnodecan.h>
#include <ul_htimer.h>
#include <ul_log.h>
#include "ul_utmalloc.h"

//======================== logging support ==========================

extern UL_LOG_CUST(ulogd_vcanet);

//---------------------------------------------------------------------------

static inline
int cmp_can_ids(canmsg_id_t *a, canmsg_id_t *b)
{
  return *a>*b?1:*a<*b?-1:0;
}

UL_LIST_CUST_DEC(vcaSDOReqList, vcaSDOFsm_t, vcaSDORequest_t,
                 requests, queue_node)

UL_LIST_CUST_DEC(vcaNetMasked, vcaNet_t, vcaNetId_t,
                 masked_ids, node.list)


GAVL_CUST_NODE_INT_DEC(vcaNetDistinct, vcaNet_t, vcaNetId_t, canmsg_id_t,
                 distinct_ids, node.distinct, id, cmp_can_ids)

GAVL_CUST_NODE_INT_IMP(vcaNetDistinct, vcaNet_t, vcaNetId_t, canmsg_id_t,
                 distinct_ids, node.distinct, id, cmp_can_ids)

GAVL_CUST_NODE_INT_DEC(vcaNetNode, vcaNet_t, vcaCANNode_t, int,
                 can_nodes, tree_node, node_number, gavl_cmp_int)

GAVL_CUST_NODE_INT_IMP(vcaNetNode, vcaNet_t, vcaCANNode_t, int,
                 can_nodes, tree_node, node_number, gavl_cmp_int)


vcaNetId_t *vcaNetId_init(vcaNetId_t *netid, canmsg_id_t id, canmsg_id_t id_mask)
{
    if(!netid){
        netid = (vcaNetId_t*)malloc(sizeof(vcaNetId_t));
        if(!netid){
            ul_logerr("memory allocation ERROR\n");
            return NULL;
        }
    }
    netid->refcnt=1;
    netid->dead=0;
    netid->masked=0;
    netid->distinct=0;
    netid->net=NULL;
    netid->id=id;
    netid->id_mask=id_mask;
    evc_tx_hub_init(&netid->tx_hub);
    return netid;
}

void vcaNetId_destroy(vcaNetId_t *netid)
{
    evc_tx_hub_done(&netid->tx_hub);
}

//===============================================================
//                        vcaCANNode_t
//===============================================================
vcaCANNode_t *vcaCANNode_init(vcaCANNode_t *cannode, int node_number)
{
    if(!cannode){
        cannode = (vcaCANNode_t*)malloc(sizeof(vcaCANNode_t));
        if(!cannode){
           ul_logerr("memory allocation ERROR\n");
           return NULL;
        }
    }
    vcaSDOFsm_init(&(cannode->sdofsm), cannode);
    cannode->node_number=node_number;
    return cannode;
}

void vcaCANNode_destroy(vcaCANNode_t *cannode)
{
    vcaSDOFsm_destroy(&(cannode->sdofsm));
}


int vcaCANNode_addSDORequest(vcaCANNode_t *cannode, vcaSDORequest_t *req)
{
    vcaSDOFsm_t *sdofsm = &(cannode->sdofsm);
    if(!sdofsm->fsm_connected){
        vcaSDOFsm_connect(sdofsm);
    }
    return vcaSDOFsm_addSDORequest(sdofsm, req);
}

/*
vcaSDOFsm_t *vcaCANNode_get_sdofsm(vcaCANNode_t *cannode)
{
    if(!cannode->sdofsm){
        cannode->sdofsm=vcaSDOFsm_init(NULL,cannode);
    }
    return cannode->sdofsm;
}
*/

//===============================================================
//                        vcaSDOFsm_t
//===============================================================
void vcaSDOFsm_msgrec_fnc(vcaSDOFsm_t *sdofsm, int event_code, canmsg_t *canmsg);

void vcaSDOFsm_timeout_fnc(unsigned long data);

vcaSDOFsm_t *vcaSDOFsm_init(vcaSDOFsm_t *sdofsm, vcaCANNode_t *cannode)
{
    if(!sdofsm){
        sdofsm = (vcaSDOFsm_t*)malloc(sizeof(vcaSDOFsm_t));
        if(!sdofsm){
            ul_logerr("memory allocation ERROR\n");
            return NULL;
        }
    }

    vcaSDOReqList_init_head(sdofsm); // list head init (field requests)
    sdofsm->cannode=cannode;
    evc_rx_hub_init(&sdofsm->rx_hub,(evc_rx_fnc_t*)vcaSDOFsm_msgrec_fnc, sdofsm);
    ul_htimer_init_detached(&sdofsm->timer);
    sdofsm->timer.function = vcaSDOFsm_timeout_fnc;
    sdofsm->timer.data = (unsigned long)sdofsm;
    sdofsm->timeout_time = 2000;
    sdofsm->fsm_connected = 0;
    sdofsm->actrq = NULL;
    return sdofsm; 
}

void vcaSDOFsm_destroy(vcaSDOFsm_t *sdofsm)
{
    vcaSDORequest_t *req;
    while((req = vcaSDOReqList_first(sdofsm)) != NULL) {
        /*vcaSDOFsm_abortSDORequest(sdofsm, req);*/
	vcaSDORequest_abort(req);
    }
}

void vcaSDOFsm_setTimeout(vcaSDOFsm_t *sdofsm)
{
    ul_htim_time_t new_time;

    ul_htimer_detach(&ul_root_htimer, &sdofsm->timer);
    ul_mstime_now(&new_time);
    new_time += sdofsm->timeout_time;
    ul_htimer_set_expire(&sdofsm->timer, new_time);
    ul_htimer_add(&ul_root_htimer, &sdofsm->timer);
}

void vcaSDOFsm_timeout_fnc(unsigned long data)
{
    vcaSDOFsm_t *sdofsm=(vcaSDOFsm_t *)data;

    ul_logerr("vcaSDOFsm_timeout_fnc() - SDO transfer timeout\n");

    vcasdo_fsm_set_error(&sdofsm->fsm, sdofsmErrTimeout);
    vcaSDOFsm_finishRequest(sdofsm);
    vcaSDOFsm_processPendingRequest(sdofsm);
}


void vcaSDOFsm_msgrec_fnc(vcaSDOFsm_t *sdofsm, int event_code, canmsg_t *canmsg)
{
    vcasdo_fsm_t *fsm=&sdofsm->fsm;
    int res;

    if(!sdofsm->fsm_connected)
        return;
  
    if(fsm->state == sdofsmRun) {
        if(vcasdo_fsm_taste_msg(fsm, canmsg)) {
            ul_htimer_detach(&ul_root_htimer, &sdofsm->timer);
            if(((fsm->state==sdofsmRun) || (fsm->state==sdofsmAbort))
              && fsm->out_msg.length) {
                res=vcaNet_send_can_msg(sdofsm->cannode->net, &fsm->out_msg);
                if(res<0)
	             vcasdo_fsm_set_error(fsm, sdofsmErrTxMsg);
                else
                     vcaSDOFsm_setTimeout(sdofsm);
            }
        }
    }

    if(fsm->state != sdofsmRun) {
        vcaSDOFsm_finishRequest(sdofsm);
        vcaSDOFsm_processPendingRequest(sdofsm);
    }

}

int vcaSDOFsm_connect(vcaSDOFsm_t *sdofsm)
{
    canmsg_id_t srvcli_port=0;
    canmsg_id_t clisrv_port=0;

    if(!sdofsm->cannode)
        return -1;
    if(!sdofsm->cannode->net)
        return -1;
    if(sdofsm->fsm_connected)
        return 1;

    /* this is very simplified for now*/
    vcasdo_init_fsm(&sdofsm->fsm, srvcli_port, clisrv_port, sdofsm->cannode->node_number);

    if(vcaNet_msg_rec_connect(sdofsm->cannode->net, &sdofsm->rx_hub, sdofsm->fsm.srvcli_cob_id, MSG_ID_MASK)<0)
        return -1;

    sdofsm->fsm_connected=1;
    return 2;
}

int vcaSDOFsm_addSDORequest(vcaSDOFsm_t *sdofsm, vcaSDORequest_t *req)
{
    vcaSDOReqList_insert(sdofsm, req);
    req->is_pending = 1;
    vcaSDOFsm_processPendingRequest(sdofsm);
    return 0;
}

int vcaSDOFsm_abortSDORequest(vcaSDOFsm_t *sdofsm, vcaSDORequest_t *req)
{
    vcasdo_fsm_t *fsm=&sdofsm->fsm;
    
    if(req->sdofsm != sdofsm)
        return -1;

    if(sdofsm->actrq == req) {
        if(fsm->state == sdofsmRun) {
	         vcasdo_fsm_set_error(fsm, sdofsmErrUserAbort);
        }
        vcaSDOFsm_finishRequest(sdofsm);
        return 1;
    } else {
        vcaSDOReqList_delete(sdofsm, req);
        req->is_pending = 0;
        return 0;
    }
}

void vcaSDOFsm_finishRequest(vcaSDOFsm_t *sdofsm)
{
    vcasdo_fsm_t *fsm=&sdofsm->fsm;
    vcaSDORequest_t *sdorq;

    if(fsm->state == sdofsmRun) {
        ul_logerr("vcaSDOFsm_finishRequest() - called when FSM active!\n");
	return;
    }

    ul_htimer_detach(&ul_root_htimer, &sdofsm->timer);

    sdorq = sdofsm->actrq;
    if(sdorq) {
        sdorq->state = fsm->state;
        sdorq->err_no = fsm->err_no;
        sdorq->node = fsm->node;

        if(sdorq->is_upload && (fsm->state == sdofsmDone)) {
            if(ul_dbuff_cpy(&sdorq->data, fsm->data.data, fsm->data.len) != fsm->data.len) {
                 sdorq->state = sdofsmError;
                 sdorq->err_no = sdofsmErrNoMemory;
            }
        }

        ul_loginf("vcaSDOFsm_finishRequest() - sdo request finished\n");

        sdorq->is_pending = 0;

        if(sdorq->finished_callback)
            sdorq->finished_callback(sdorq);
    }

    vcasdo_fsm_idle(fsm);

    sdofsm->actrq = NULL;
}

void vcaSDOFsm_processPendingRequest(vcaSDOFsm_t *sdofsm)
{
    vcasdo_fsm_t *fsm=&sdofsm->fsm;
    int res;
    vcaSDORequest_t *sdorq;

    
    if(fsm->state != sdofsmIdle) return;
    if(sdofsm->actrq) return; 
    
    do {
        if(!sdofsm->fsm_connected || !sdofsm->cannode || !sdofsm->cannode->net){
            return;
        }

        sdorq = vcaSDOReqList_cut_first(sdofsm);
        if(!sdorq) return;
        sdofsm->actrq = sdorq;

        /* setup FSM for request parameters */
        if(sdorq->is_upload) {
            ul_loginf("vcaSDOFsm_processPendingRequest() - starting SDO upload processing\n");
            vcasdo_fsm_upload(fsm, fsm->node, sdorq->index, sdorq->subindex, 
                                  fsm->srvcli_cob_id, fsm->clisrv_cob_id);
        }
        else {
            ul_loginf("vcaSDOFsm_processPendingRequest() - starting SDO download processing\n");
            vcasdo_fsm_download(fsm, &sdorq->data, fsm->node, sdorq->index, sdorq->subindex,
                                    fsm->srvcli_cob_id, fsm->clisrv_cob_id);
        }

        if(fsm->state == sdofsmRun) {
            res=vcaNet_send_can_msg(sdofsm->cannode->net, &fsm->out_msg);
            if(res<0){
	         vcasdo_fsm_set_error(fsm, sdofsmErrTxMsg);
            }
            else{
                 vcaSDOFsm_setTimeout(sdofsm);
            }
        }

        if(fsm->state == sdofsmRun)
            return;

        vcaSDOFsm_finishRequest(sdofsm);
    } while(1);
}

//===============================================================
//                        vcaNet_t
//===============================================================
void vcaNet_msgrec_fnc(vcaNet_t *net, int event_code, canmsg_t *canmsg);

vcaNet_t *vcaNet_init(vcaNet_t *net)
{
      if(!net){
          net = (vcaNet_t*)malloc(sizeof(vcaNet_t));
          if(!net) {
              ul_logerr("memory allocation ERROR\n");
              return NULL;
          }
      }

      vcaNetMasked_init_head(net); // list init
      vcaNetDistinct_init_root_field(net); // GAVL init
      vcaNetNode_init_root_field(net); // GAVL init
      evc_rx_hub_init(&net->rx_hub,(evc_rx_fnc_t*)vcaNet_msgrec_fnc,net);
      net->canfdnode=NULL;
      return net; 
}

void vcaNet_destroy(vcaNet_t *net)
{
      vcaCANNode_t *cannode;
      vcaNetId_t *netid;

      while((netid=vcaNetMasked_cut_first(net)) != NULL) {
          vcaNetId_destroy(netid);
	  free(netid);
      }
      while((netid=vcaNetDistinct_cut_first(net)) != NULL) {
          vcaNetId_destroy(netid);
	  free(netid);
      }
      while((cannode=vcaNetNode_cut_first(net)) != NULL) {
          vcaCANNode_destroy(cannode);
	  free(cannode);
      }
      evc_rx_hub_done(&net->rx_hub);
      
      if(net->canfdnode != NULL) {
          vcaFdNodeCan_destroy(net->canfdnode);
          free(net->canfdnode);
      }
}

void vcaNet_printRegisteredIDs(vcaNet_t *net)
{
    vcaNetId_t *netid;

    ul_loginf("List of distinct filtered IDs\n");
    gavl_cust_for_each(vcaNetDistinct, net, netid){
        ul_loginf("  ID 0x%lx mask 0x%lx\n", netid->id, netid->id_mask);
    }
    ul_loginf("List of masked filtered IDs\n");
    ul_list_for_each(vcaNetMasked, net, netid){
        ul_loginf("  ID 0x%lx mask 0x%lx\n", netid->id, netid->id_mask);
    }
}    


struct vcaFdNodeCan_t* vcaNet_connect_device(vcaNet_t *net, struct vcaFdNodeCan_t *canfdnode, const char *device_name)
{
    if(!canfdnode) canfdnode = vcaFdNodeCan_init(NULL, device_name);
    net->canfdnode=canfdnode;
    if(net->canfdnode){
        evc_prop_fnc_t *prop;
        evc_link_t *link;
        link=evc_link_new();
        prop=&vca_msg_propagate_event;
        evc_link_connect(link, &net->canfdnode->tx_hub, &net->rx_hub, prop);
        evc_link_dec_refcnt(link);
    }
    return canfdnode;
}

// TODO: comment, what is event_code param good for
void vcaNet_msgrec_fnc(vcaNet_t *net, int event_code, canmsg_t *canmsg)
{
    vcaNetId_t *netid;
    
    // TODO: Rough messages
    /*
       enumerate all connected clients and check if they are configured to
       send rough CAN messages. If yes pickle message and send it to the client.
     */
    /*
    vcaCANNode_t *nd; 
    //clog(LOG_TRASH, "vcaNet_msgrec_fnc():\n");
    gavl_cust_for_each(vcaNetNode, net, nd) {
        clog(LOG_TRASH, "vcaNet_msgrec_fnc(): traversing node #%i\n", nd->node_number);
    }
    */ 
    netid=vcaNet_find_id(net, canmsg->id&MSG_ID_MASK, 0);
    if(netid)
        evc_tx_hub_emit(&netid->tx_hub, event_code, canmsg);

    netid=vcaNetMasked_first(net);
    while(netid){
        if(((netid->id^canmsg->id)&netid->id_mask)==0){
            evc_tx_hub_emit(&netid->tx_hub, event_code, canmsg);
        }
        netid=vcaNetMasked_next(net,netid);
    }
}

vcaNetId_t *vcaNet_find_id(vcaNet_t *net, canmsg_id_t id, int create)
{
    vcaNetId_t *netid;
    netid=vcaNetDistinct_find(net, &id);
    if(!netid){
        if(!create)
            return NULL;
        netid=vcaNetId_init(NULL, id, ~0);
        if(!net){
            ul_logerr("cannot create vcaNetId\n");
            return NULL;
        }
        netid->distinct=1;
        netid->net=net;
        vcaNetDistinct_insert(net, netid);
    }
    return netid;
}

vcaNetId_t *vcaNet_find_masked_id(vcaNet_t *net, canmsg_id_t id, canmsg_id_t id_mask, int create)
{
    vcaNetId_t *netid;

    ul_list_for_each(vcaNetMasked, net, netid){
        if((netid->id==id)&&(netid->id_mask==id_mask))
            return netid;
    }

    netid=vcaNetId_init(NULL, id, id_mask);
    if(!netid)
        return NULL;

    netid->masked=1;
    vcaNetMasked_insert(net,netid);
    return netid;
}

vcaCANNode_t *vcaNet_find_cannode(vcaNet_t *net, int node_number, int create)
{
    vcaCANNode_t *cannode;
    cannode=vcaNetNode_find(net, &node_number);
    if(!cannode){
        if(!create)
            return NULL;
        cannode=vcaCANNode_init(NULL, node_number);
        if(!net){
            ul_logerr("cannot create vcaCANNode\n");
            return NULL;
        }
        cannode->net=net;
        vcaNetNode_insert(net, cannode);
    }
    return cannode;
}

int vcaNet_msg_rec_connect(vcaNet_t *net, evc_rx_hub_t *rx_hub, canmsg_id_t id, canmsg_id_t id_mask)
{
    evc_prop_fnc_t *prop;
    vcaNetId_t *netid;
    evc_link_t *link;
    int ret;

    if((id_mask&MSG_ID_MASK)==MSG_ID_MASK){
        netid=vcaNet_find_id(net, id, 1);
        if(!netid)
            return -1;
    }else{
        netid=vcaNet_find_masked_id(net, id&id_mask, id_mask, 1);
        if(!netid)
            return -1;
    }
    link=evc_link_new();
    prop=&vca_msg_propagate_event;
    ret=evc_link_connect(link, &netid->tx_hub, rx_hub, prop);
    evc_link_dec_refcnt(link);
    return ret;
}

int vcaNet_send_can_msg(vcaNet_t *net, const canmsg_t *msg)
{
    do {
        if(!net) break;
        if(!net->canfdnode) break;
        vca_send_msg_seq(net->canfdnode->vcah, (canmsg_t*)msg, 1);
        return 0;
    } while(0);
    ul_logerr("vcaNet_send_can_msg() - ERROR sending CAM message\n");
    return -1;
}

//===============================================================
//                        vcaSDORequest_t
//===============================================================
vcaSDORequest_t *vcaSDORequest_init(vcaSDORequest_t *req, 
    int is_upload,
    int index, int subindex,
    void *data, int data_len)
{
    if(!req){
        req = (vcaSDORequest_t*)malloc(sizeof(vcaSDORequest_t));
        if(!req){
            ul_logerr("memory allocation ERROR\n");
            return NULL;
        }
    }
    
    req->sdofsm = NULL;
    req->index = index;
    req->subindex = subindex;
    req->is_upload = is_upload;
    req->is_pending = 0;
    ul_dbuff_init(&(req->data), 0);
    ul_dbuff_cpy(&(req->data), data, data_len);
    return req;
}

void vcaSDORequest_destroy(vcaSDORequest_t *req)
{
    if(req->is_pending)
         vcaSDORequest_abort(req);

    ul_dbuff_destroy(&(req->data));
}

void vcaSDORequest_abort(vcaSDORequest_t *req)
{
    if(!req->sdofsm)
        return;

    vcaSDOFsm_abortSDORequest(req->sdofsm, req);
}
