#ifndef __RTL__

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>

//#include "eds_utils.h"

#else /*__RTL__*/

#include <rtl.h>
#include <string.h>
#include <signal.h>
#include <posix/unistd.h>

#define printf rtl_printf
#define strtol simple_strtol
#define strtoll simple_strtoll
#define strtoul simple_strtoul
#define strtoull simple_strtoull

#endif /*__RTL__*/

#include "ul_utmalloc.h"
#include <ul_log.h>
#include <sui_event.h>
#include <sui_dinfo.h>
#include <sui_dinfo_dbuff.h>
#include <sui_dievc.h>
#include "can_vca.h"
#include "vca_od.h"
#include "vcasdo_msg.h"
#include "vca_hwmod.h"

#ifdef CONFIG_OD_GSA
    #include "ul_gsa.h"
    #include "ul_gsacust.h"
#else
    #include "ul_gavl.h"
    #include "ul_gavlcust.h"
#endif
//static const char* od_item_type2str(int type);
//static int get_object_data_size(vcaod_object_t *o, uint32_t *abort_code);

//====================== logging support =======================

extern UL_LOG_CUST(ulogd_vca);

//=================================================================
typedef struct data_type_def_t {
    const char *name;
    int size;
} data_type_def_t;

enum {  data_type_UNKNOWN = 0, data_type_BOOLEAN, 
        data_type_INTEGER8, data_type_INTEGER16, data_type_INTEGER32, 
        data_type_UNSIGNED8, data_type_UNSIGNED16, data_type_UNSIGNED32,
        data_type_REAL32,
        data_type_VISIBLE_STRING, data_type_OCTET_STRING, data_type_UNICODE_STRING,
        data_type_TIME_OF_DAY, data_type_TIME_DIFERENCE,
        data_type_reserved1,
        data_type_DOMAIN,
        data_type_INTEGER24,
        data_type_REAL64,
        data_type_INTEGER40, data_type_INTEGER48, data_type_INTEGER56, data_type_INTEGER64, 
        data_type_UNSIGNED24,
        data_type_reserved2,
        data_type_UNSIGNED40, data_type_UNSIGNED48, data_type_UNSIGNED56, data_type_UNSIGNED64,
        data_type_reserved3, data_type_reserved4, data_type_reserved5, data_type_reserved6,
        data_type_PDO_COMMUNICATION_PARAMETER, data_type_PDO_MAPPING,
        data_type_SDO_PARAMETER,
        data_type_IDENTITY
};

static data_type_def_t data_types[] = 
    {{"UNKNOWN", -1}, {"BOOLEAN", 1}, 
     {"INTEGER8", 1}, {"INTEGER16", 2}, {"INTEGER32", 4}, 
     {"UNSIGNED8", 1}, {"UNSIGNED16", 2}, {"UNSIGNED32", 4},
     {"REAL32", 4},
     {"VISIBLE_STRING", 0}, {"OCTET_STRING", 0}, {"UNICODE_STRING", 0},
     {"TIME_OF_DAY", -1}, {"TIME_DIFERENCE", -1},
     {"reserved", -1},
     {"DOMAIN", -1},
     {"INTEGER24", 3},
     {"REAL64", 8},
     {"INTEGER40", 5}, {"INTEGER48", 6}, {"INTEGER56", 7}, {"INTEGER64", 8}, 
     {"UNSIGNED24", 3},
     {"reserved", -1},
     {"UNSIGNED40", 5}, {"UNSIGNED48", 6}, {"UNSIGNED56", 7}, {"UNSIGNED64", 8},
     {"reserved", -1}, {"reserved", -1}, {"reserved", -1}, {"reserved", -1},
     {"PDO_COMMUNICATION_PARAMETER", -1}, {"PDO_MAPPING", -1},
     {"SDO_PARAMETER", -1},
     {"IDENTITY", -1}
    };
static int data_types_cnt = sizeof(data_types) / sizeof(data_types[0]);

//==================== vca OD iplementation =======================
inline int vcaod_cmp_fnc(const unsigned *a, const unsigned *b)
{
  if (*a > *b) return 1;
  if (*a < *b) return -1;
  return 0;
}

#ifdef CONFIG_OD_GSA
//GSA_CUST_DEC(vcaod, vcaod_root_t, vcaod_object_t, unsigned,
//	my_root, index, vcaod_cmp_fnc)
 
GSA_CUST_IMP(vcaod, vcaod_root_t, vcaod_object_t, unsigned,
	my_root, index, vcaod_cmp_fnc, GSA_FANY)
#else
//GAVL_CUST_NODE_INT_DEC(vcaod, vcaod_root_t, vcaod_object_t, unsigned,
//	my_root, my_node, index, vcaod_cmp_fnc)

GAVL_CUST_NODE_INT_IMP(vcaod, vcaod_root_t, vcaod_object_t, unsigned,
	my_root, my_node, index, vcaod_cmp_fnc)
#endif

//=================================================================
vcaod_object_t* od_item_init(vcaod_object_t *item)
{
    if(!item) item = (vcaod_object_t*)malloc(sizeof(vcaod_object_t));
    if(item) {
        memset(item, 0, sizeof(*item));
        item->subindex = -1;
        ul_dbuff_init(&item->value, 0);
        item->dinfo = NULL;
    }
    return item;
}
//.................................................................
static void od_item_destroy(vcaod_object_t *o)
{
    if(o) {
        #ifdef _DEBUG_FULL
        ul_logdeb("od_item_destroy(): object [%04x:%02x]\n", o->index, o->subindex);
        #endif
        if(o->dinfo) {
            if(o->flags & VCAOD_OBJECT_FLAG_WEAK_DINFO) sui_dinfo_dec_refcnt(o->dinfo);
            else sui_dinfo_del_weakptr(o->dinfo, &(o->dinfo));
        }
        if(o->subobjects) {
            int i;
            for(i=0; i<o->subcnt; i++) {
                vcaod_object_t *o1 = o->subobjects + i;
                if(o1) od_item_destroy(o1);
            }
            free(o->subobjects);
        }
        ul_dbuff_destroy(&o->value);
    }
}
//.................................................................
int od_item_set_value_u64(vcaod_object_t *item, uint64_t val)
{
    int ret = 0;
    int size;
    ret = vcaod_get_object_data_size(item, 0);
    if(ret < 0) return ret; //error
    
    size = ret;
    ret = ul_dbuff_set_len(&(item->value), size);
    if(ret < 0) return ret; //error
    memset(item->value.data, 0, item->value.len);
    if(size > sizeof(val)) size = sizeof(val);
    memcpy(item->value.data, &val, size);
    //ul_dbuff_fix_endianning(&(item->value));
    return ret;
}
//.................................................................
int od_item_set_value_as_str(vcaod_object_t *item, const char* valstr)
{
    int ret = 0;
    if(!item) return ret;
    if(item->data_type == data_type_VISIBLE_STRING) {
        int size = strlen(valstr);
        ret = ul_dbuff_set_len(&(item->value), size);
        if(ret > 0) memcpy(item->value.data, valstr, item->value.len);
    }
    else if(item->data_type >= data_type_BOOLEAN && item->data_type <= data_type_INTEGER32) {
        long i = strtol(valstr, NULL, 0);
        ret = od_item_set_value_u64(item, i);
    }
    else if(item->data_type >= data_type_UNSIGNED8 && item->data_type <= data_type_UNSIGNED32) {
        unsigned long i = strtoul(valstr, NULL, 0);
        ret = od_item_set_value_u64(item, i);
    }
    else if(item->data_type == data_type_INTEGER24 || (item->data_type >= data_type_INTEGER40 && item->data_type <= data_type_INTEGER64)) {
        //printf("od_item_set_value_as_str(): item_data_type: 0x%x, value: '%s'\n", item->data_type, valstr);
        long long i = strtoll(valstr, NULL, 0);
        ret = od_item_set_value_u64(item, i);
    }
    else if(item->data_type == data_type_UNSIGNED24 || (item->data_type >= data_type_UNSIGNED40 && item->data_type <= data_type_UNSIGNED64)) {
        unsigned long long i = strtoull(valstr, NULL, 0);
        ret = od_item_set_value_u64(item, i);
    }
    return ret;
}
//-----------------------------------------------------------------
void vcaod_od_free(vcaod_root_t *odroot)
{
    vcaod_object_t *o;
    if(!odroot) return;
    while((o = vcaod_first(odroot)) != NULL) {
        vcaod_delete(odroot, o);
        od_item_destroy(o);
        free(o);
    }
}
//-------------------------------------------------------------------
const char* od_item_type2str(int type)
{
    const char *pc;
    switch(type) {
        case vcaod_object_type_DOMAIN: pc = "DOMAIN"; break;
        case vcaod_object_type_DEFTYPE: pc = "DEFTYPE"; break;
        case vcaod_object_type_DEFSTRUCT: pc = "DEFSTRUCT"; break;
        case vcaod_object_type_VAR: pc = "VAR"; break;
        case vcaod_object_type_ARRAY: pc = "ARRAY"; break;
        case vcaod_object_type_RECORD: pc = "RECORD"; break;
        default: pc = "UNKNOWN";
    }
    return pc;
}
//...................................................................
static void dump_item(const vcaod_object_t *item)
{
    char indent = '\t';
    if(item->subindex < 0) indent = ' ';
    //ul_loginf("%c\tParameterName='%s'\n", indent, item->name);
    printf("%c\tParameterName='%s'\n", indent, item->name);
    if(item->subcnt > 0) {
        //ul_loginf("%c\tSubNumber=%i\n", indent, item->subcnt);
        printf("%c\tSubNumber=%i\n", indent, item->subcnt);
    }
    else {
        int type;
        int i;
        unsigned char *buf;
        const char *pc = od_item_type2str(item->object_type);
	//ul_loginf("%c\tObjectType=%02x(%s)\n", indent, item->object_type, pc);
        printf("%c\tObjectType=%02x(%s)\n", indent, item->object_type, pc);
        type = item->data_type;
        if(type >= data_types_cnt) type = 0;
        //ul_loginf("%c\tDataType=%04x(%s)\n", indent, item->data_type, data_types[type].name);
        printf("%c\tDataType=%04x(%s)\n", indent, item->data_type, data_types[type].name);
        switch(item->access) {
            case vcaod_access_RW: pc = "rw"; break;
            case vcaod_access_RWR: pc = "rwr"; break;
            case vcaod_access_RWW: pc = "rww"; break;
            case vcaod_access_RO: pc = "ro"; break;
            case vcaod_access_WO: pc = "wo"; break;
            case vcaod_access_CONST: pc = "const"; break;
            default: pc = "unknown";
        }
        //ul_loginf("%c\tAccessType=%s\n", indent, pc);
        printf("%c\tAccessType=%s\n", indent, pc);
        //ul_loginf("%c\tPDOMapping=%i\n", indent, item->pdo_mapping);
        printf("%c\tPDOMapping=%s\n", indent, (item->flags & VCAOD_OBJECT_FLAG_PDO_MAPPING)? "yes": "no");
        printf("%c\tValue= ", indent);
        buf = item->value.data;
        for(i=0; i<item->value.len; i++) {
            if(i == 0) printf("dbuff [");
            else       printf(" ");
            printf("%02x", buf[i]);
        }
        printf("]\n");
    }
}
//...................................................................
void vcaod_dump_od(vcaod_root_t *odroot)
{
    vcaod_object_t *item;
    //gavl_cust_for_each(vcaod, odroot, item) { // only in C99 mode
    //for(item=vcaod_first(odroot); item; item=vcaod_next(odroot, item)) { // not applicable on GSA
    vcaod_it_t it;
    ul_for_each_it(vcaod, odroot, it) { // EXAMPLE list/gavl/GSA iterator ussage
        item = vcaod_it2item(&it);
        //ul_loginf("[%04x]\n", item->index);
        printf("[%04x]\n", item->index);
        dump_item(item);
        if(item->subcnt) {
            int i;
            // item->subobjects[subcnt] can be optional FF subindex
            for(i=0; i<item->subcnt; i++) {
                int six = item->subobjects[i].subindex;
                // skip subindicies not specified in EDS
                if(six < 0) continue;
                //ul_loginf("\n");
                //ul_loginf("\tsubindex[%02x]\n", six);
                printf("\n\tsubindex[%02x]\n", six);
                dump_item(item->subobjects + i);
            }
        }
    }
}
//-------------------------------------------------------------------
vcaod_object_t* vcaod_find_object(vcaod_root_t *odroot, unsigned ix, unsigned subix, uint32_t *abort_code)
{
    vcaod_object_t *o = vcaod_find(odroot, &ix);
    if(abort_code) *abort_code = 0xFFFFFFFF;
    if(!o) {
        ul_logmsg("vcaod_find_object(): object %04x:%02x index not found in OD\n", ix, subix);
        if(abort_code) *abort_code = 0x6020000;
        return NULL;
    }
    if(o->subcnt == 0) return o;
    if(o->object_type != vcaod_object_type_ARRAY) {
        int s = -1;
        if(subix == 0xFF) s = o->subcnt - 1;
        else if(subix < o->subcnt) s = subix;
        if(s == -1) {
            ul_logmsg("vcaod_get_value(): object %04x:%02x subindex not found in OD\n", ix, subix);
            if(abort_code) *abort_code = 0x6090011;
            return NULL;
        }
        if(o->subobjects[s].subindex != subix) {
            ul_logmsg("vcaod_get_value(): object %04x:%02x subobject[%i] does not have subindex %x\n", ix, subix, s, subix);
            if(abort_code) *abort_code = 0x6090011;
            return NULL;
        }
        o = o->subobjects + s;
    }
    return o;
}
//..........................................................................
int vcaod_get_object_data_size(const vcaod_object_t *o, uint32_t *abort_code)
{
    int dsize = -1;
    if(o->data_type < data_types_cnt) dsize = data_types[o->data_type].size;
    if(dsize == -1) {
        if(abort_code) {
            ul_logerr("get_object_size(): [%04x:%02x] size of data type 0x%x not known\n", o->index, o->subindex, o->data_type);
            *abort_code = 0x8000000;
        }
    }
    return dsize;
}
//..........................................................................
#define MIN(a, b) ((a < b)? a: b)
int vcaod_get_value(const vcaod_object_t *o, int array_index, void *buff, int buff_len, uint32_t *abort_code)
{
    int dsize;
    if(abort_code) *abort_code = 0xFFFFFFFF; // not CANopen defined error
    //ul_logdeb("vcaod_get_value(): reading object %04x:%02x value\n", ix, subix);
    if(!o) {
        ul_logmsg("vcaod_get_value(): object is NULL\n");
        return -1;
    }
    // check write only
    if(o->access == vcaod_access_WO) {
        ul_logmsg("vcaod_set_value(): [%04x:%02x] attempt to read WRITE ONLY object\n", o->index, o->subindex);
        if(abort_code) *abort_code = 0x6010001;
        return -1;
    }
    
    // get read object size
    dsize = vcaod_get_object_data_size(o, abort_code);
    if(dsize < 0) return -1;
    if(dsize > buff_len) {
        ul_logerr("vcaod_get_value(): object %04x:%02x size of data (%i) exceeds size of buffer (%i)\n", 
                       o->index, o->subindex, dsize, buff_len);
        return -1;
    }
    if(o->dinfo) {
        // use dinfo if it does exists
        // TODO: support reading other objects than long via dinfo
        // sui_wr_long() fires object change if necessary
        long val;
        if(sui_rd_long(o->dinfo, 0, &val) == SUDI_DATA_OK) {
            ul_logdeb("vcaod_get_value(): object value read using dinfo\n");
            memset(buff, 0, buff_len);
            // TODO: endianing
            dsize = MIN(dsize, sizeof(val));
            memcpy(buff, &val, dsize);
        }
        else {
            ul_logdeb("vcaod_get_value(): ERROR reading object value using dinfo\n");
            if(abort_code) *abort_code = 0x08000020; // "Data cannot be transferred or stored to the application.
            return -1;
        }
        return dsize;
    }
    else {
        int dstart;
        int is_array;
        const ul_dbuff_t *db;
        dstart = -1;
        is_array = (o->object_type == vcaod_object_type_ARRAY);
        if(is_array) {
            if(array_index < o->subcnt) {
                dstart = array_index * dsize;
            }
            else {
                ul_logerr("vcaod_get_value(): object [%04x:%02x] array index [%i] out of bounds (%i)\n", 
                               o->index, o->subindex, array_index, o->subcnt);
                if(abort_code) *abort_code = 0x6090011;
                return -1;
            }
        }
        else {
            dstart = 0;
        }
        db = &(o->value);
        if(dstart + dsize > db->len) {
            ul_logerr("vcaod_get_value(): object %04x:%02x dbuff len(%li) shorter than object size(%i)\n", 
                            o->index, o->subindex, db->len, dstart + dsize);
            ul_logerr("\tpossible reason: OD entry wasn't initialized, try to write to it first.\n");
            if(abort_code) *abort_code = 0x8000000;
            return -1;
        }
        memset(buff, 0, buff_len);
        memcpy(buff, db->data + dstart, dsize);
        return dsize;
    }
    return -1;
}
//-------------------------------------------------------------------
// TODO: fatal repair return value meaning see. header
int vcaod_set_value(vcaod_object_t *o, int array_index, const void *buff, int buff_len, uint32_t *abort_code)
{
    //vcaod_object_t *o = vcaod_find_object(odroot, ix, subix, abort_code);
    int dsize;
    if(abort_code) *abort_code = 0xFFFFFFFF; // not CANopen defined error
    dsize = vcaod_get_object_data_size(o, abort_code);
    if(dsize < 0) return -1;
    ul_logdeb("vcaod_set_value(): set object %04x:%02x value\n", o->index, o->subindex);
    if(!o) {
        ul_logerr("vcaod_set_value(): object is NULL\n");
        return -1;
    }
    // check read only
    if(o->access < vcaod_access_RW) {
        ul_logmsg("vcaod_set_value(): [%04x:%02x] attempt to write READ ONLY object\n", 
                        o->index, o->subindex);
        if(abort_code) *abort_code = 0x6010002;
        return -1;
    }

    if(o->dinfo) {
        // TODO: support updating other objects than long via dinfo
        // use dinfo if it does exists
        // sui_wr_long() fires object change if necessary
        if(sui_wr_long(o->dinfo, 0, buff) == SUDI_DATA_OK) {
            ul_logdeb("vcaod_set_value(): object value set using dinfo\n");
        }
        else {
            ul_logdeb("vcaod_set_value(): ERROR seting object value using dinfo\n");
            if(abort_code) *abort_code = 0x08000020; // "Data cannot be transferred or stored to the application.
            return -1;
        }
        return sizeof(long);
    }
    else {
        int is_array;
        int sz;
        int dstart;
        ul_dbuff_t *db;
        
        is_array = (o->object_type == vcaod_object_type_ARRAY);
        sz = dsize;
        dstart = 0;
        if(is_array) {
            sz *= o->subcnt;
            if(array_index < o->subcnt) {
                dstart = array_index * dsize;
            }
            else {
                ul_logerr("vcaod_set_value(): object [%04x:%02x] array index [%i] out of bounds (%i)\n", 
                                o->index, o->subindex, array_index, o->subcnt);
                if(abort_code) *abort_code = 0x6090011;
                return -1;
            }
        }
        db = &(o->value);
        ul_dbuff_set_len(db, sz);
        if(dstart + dsize > db->len) {
            ul_logerr("vcaod_set_value(): object %04x:%02x dbuff len (%li) exceded (%i)\n", 
                            o->index, o->subindex, db->len, dstart + dsize);
            if(abort_code) *abort_code = 0x8000000;
            return -1;
        }
        memset(db->data + dstart, 0, dsize);
        // TODO: endianing
        dsize = MIN(buff_len, dsize); // stored data can be shorter than object size
        memcpy(db->data + dstart, buff, dsize);
        return dsize;
    }
    return -1;
}
//-------------------------------------------------------------------
sui_dinfo_t *vcaod_get_dinfo_ref(vcaod_object_t *obj, int create_weak)
{
    sui_dinfo_t *d = obj->dinfo;
    
    if(d){
        /* dinfo already exists, it is either HW type or weak */
        sui_dinfo_inc_refcnt(d);
	return d;
    }
    if(!create_weak) return NULL;

    d = sui_dinfo_dbuff_create(&obj->value, 0);
    if(d == NULL) return NULL;
    ul_logdeb("\tCreated WEAK dbuff dinfo (%p) for object %04x:%02x.\n", 
                     d, obj->index, obj->subindex);
    obj->dinfo = d;
    obj->flags |= VCAOD_OBJECT_FLAG_WEAK_DINFO;
    sui_dinfo_add_weakptr(obj->dinfo, &(obj->dinfo));
    return d;
}   
//-------------------------------------------------------------------
/*
void vcaod_connect_hw_dinfos_to_OD(vcaod_root_t *odroot)
{
    int i;
    ul_loginf("--- connecting HW dinfos to OD objects ---\n");
    for(i=0; i<vcaHwModule_getDinfoCnt(); i++) {
        vcaod_object_t *o;
        int ix, six;
        uint32_t err_code;
        uint32_t u = vcaHwModule_getDinfoObjectIndex(i);
        ix = u / 0x100; six = u % 0x100;
        o = _vcaod_find_object(odroot, ix, six, &err_code);
        if(!o) {
            ul_logerr("vcaod_connect_hw_dinfos_to_OD(): [ix=%i] cann't find object %04x:%02x in OD\n", i, ix, six);
            ul_logerr("\t%s\n", vcasdo_abort_msg(err_code));
            continue;
        }
        if(o->dinfo) {
            ul_logerr("vcaod_connect_hw_dinfos_to_OD(): object %04x:%02x already has dinfo asigned\n", ix, six);
            continue;
        }
        o->dinfo = vcaHwModule_getDinfoRef(i);
        o->flags &= ~VCAOD_OBJECT_FLAG_WEAK_DINFO;        
        ul_logdeb("vcaod_connect_hw_dinfos_to_OD(): added HW dinfo (%p) to object %04x:%02x\n", o->dinfo, ix, six);
    }
} 
*/  
//-------------------------------------------------------------------
