//#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <ul_log.h>

#include "vca_pollfdnodecan.h"

//====================== logging support =======================

extern UL_LOG_CUST(ulogd_vcapollfdnodecan);

static void fatal_error(const char *format, ...)
{
    va_list ap;
    va_start(ap, format); 
    ul_vloglev(UL_LOGL_FATAL, format, ap);
    va_end(ap);
  #ifndef __RTL__
    exit(EXIT_FAILURE);
  #else /*__RTL__*/
    while (1) {
        /*Block all other activities*/
	pthread_wait_np ();
    }
  #endif /*__RTL__*/
}
//=================================================================

typedef void vca_msg_propagate_event_fnc_t(void *context, int event_code, canmsg_t *canmsg);


int vca_msg_propagate_event(evc_link_t *link, va_list args)
{
  void *context;
  vca_msg_propagate_event_fnc_t *fnc;

  if(link->dead){
    if(link->standalone && link->dst.standalone.weakptr){
      *link->dst.standalone.weakptr=NULL;
      link->dst.standalone.weakptr=NULL;
    }
    return 0;
  }
  if(link->standalone){
    fnc=(vca_msg_propagate_event_fnc_t*) link->dst.standalone.rx_fnc;
    context=link->dst.standalone.context;
  }else{
    if(!link->dst.multi.hub) return 0;
    fnc=(vca_msg_propagate_event_fnc_t*) link->dst.multi.hub->rx_fnc;
    context=link->dst.multi.hub->context;
  }
  if(fnc){
    int code=va_arg(args, int);
    canmsg_t *canmsg=va_arg(args, canmsg_t *);
    /*clog(LOG_DEB, "code = %d canmsg = %p\n", code, canmsg);*/
    fnc(context, code, canmsg);
  }
  return 0;
}

static void vcaFdNodeCan_ready(struct vcaFdNode_t *nd, int event_type, struct vcaFdPoll_t *p)
{
    char msgstr[100];
    canmsg_t canmsg;
    int event_code=0;
    int res;
    vcaFdNodeCan_t *fd = UL_CONTAINEROF(nd, vcaFdNodeCan_t, fdnode);
    res=vca_rec_msg_seq(fd->vcah, &canmsg, 1);
    if(res==1){
      vca_msg2str(&canmsg, msgstr, sizeof(msgstr));
      ul_logdeb("new CAN message: %s\n", msgstr);
      if(canmsg.flags&MSG_RTR){
        event_code=VCA_MSG_EVCODE_MSGRX_RTR;
      }
      else{
        event_code=VCA_MSG_EVCODE_MSGRX_STD;
      }
      evc_tx_hub_emit(&fd->tx_hub, event_code, &canmsg);
      //clog(LOG_DEB, "emit OK\n");
    }else{
      ul_logdeb("vcaFdNodeCan_ready: vca_rec_msg_seq returned %d\n", res);
    }
}
 
vcaFdNodeCan_t* vcaFdNodeCan_init(vcaFdNodeCan_t *nd, const char *dev_name)
{
    vca_handle_t vcah;
    int res;
    if(nd == NULL) {
        nd = (vcaFdNodeCan_t*)malloc(sizeof(vcaFdNodeCan_t));
        if(!nd) fatal_error("memory allocation ERROR\n");
    }
    
    ul_logdeb("opening device '%s'\n", dev_name);
    res = vca_open_handle(&vcah, dev_name, "", VCA_O_NONBLOCK);
    if(res < 0) {
        ul_logerr("ERROR open device '%s': %s\n", dev_name, strerror(errno));
    }
    else {
        vcaFdNode_init(&(nd->fdnode), vca_h2fd(vcah), VCAFD_RD);       
        nd->vcah = vcah;
        nd->fdnode.ready = vcaFdNodeCan_ready;
    }
    evc_tx_hub_init(&nd->tx_hub);
    return nd;
}

void vcaFdNodeCan_destroy(vcaFdNodeCan_t *nd)
{
    evc_tx_hub_done(&nd->tx_hub);
    vcaFdNode_destroy(&(nd->fdnode));
}
